/*
 * Copyright (C) 2007-2011  The Paparazzi Team
 *
 * This file is part of paparazzi.
 *
 * paparazzi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * paparazzi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with paparazzi; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/**
 * @file subsystems/navigation/common_flight_plan.h
 * Common flight_plan functions shared between fixedwing and rotorcraft.
 */

#ifndef COMMON_FLIGHT_PLAN_H
#define COMMON_FLIGHT_PLAN_H

//#include "std.h"
#include <stdint.h>

/** In s */
extern uint16_t stage_time, block_time;

/* The modification of the following variables do not affect
 * the behavior of the flight plan 
 */
extern uint8_t nav_stage, nav_block;
extern uint8_t last_block, last_stage;

/* function to initiallize the vcisl */
void init_function(void);

/** needs to be implemented by fixedwing and rotorcraft seperately */
void nav_init_stage(void);

void nav_init_block(void);
void nav_goto_block(uint8_t block_id);

/* Getter */
uint8_t get_nav_block();
uint8_t get_nav_stage();
uint8_t get_last_block();
uint8_t get_last_stage();

/* Setter */
void set_nav_block(uint8_t b);
void set_nav_stage(uint8_t s);

#define InitStage() nav_init_stage();

#define Block(x) case x: set_nav_block(x); // Modified
void NextBlock(); // Modified
#define GotoBlock(b) nav_goto_block(b)

#define Stage(s) case s: set_nav_stage(s); // Modified
void NextStage(); // Modified
#define NextStageAndBreak() { NextStage(); break; }
#define NextStageAndBreakFrom(wp) { last_wp = wp; NextStageAndBreak(); }

#define Label(x) label_ ## x:
#define Goto(x) { goto label_ ## x; }
void Return(uint8_t x); // Modified

#define And(x, y) ((x) && (y))
#define Or(x, y) ((x) || (y))
#define Min(x,y) (x < y ? x : y)
#define Max(x,y) (x > y ? x : y)
#define LessThan(_x, _y) ((_x) < (_y))
#define MoreThan(_x, _y) ((_x) > (_y))

/** Time in s since the entrance in the current block */
#define NavBlockTime() (block_time)

/** Functions generated by the flight plan */
void on_enter_block(unsigned char);
void on_exit_block(unsigned char);
_Bool forbidden_deroute(unsigned char, unsigned char);

// constant define in the flight_plan.h
extern const uint8_t nb_blocks;

#ifndef GEN
#include "flight_plan.h"
#endif

#endif /* COMMON_FLIGHT_PLAN_H */
